{# CONTEXT = [home_business_name]#}
<div class="email-body">
    Hi,

    <p>Thanks for for ordering from {{ home_business_name }}.</p>
    {% block content %}{% endblock %}
    <p>If you have any queries please let us know.</p>

    <p>Kind Regards,<br>
        {{ home_business_name }}</p>
</div>

